<?php

namespace Drupal\Tests\entity_access_by_role_field\Functional;

use Drupal\Tests\taxonomy\Traits\TaxonomyTestTrait;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\entity_access_by_role_field\Helper\Constants;

/**
 * This class provides access tests for entity_access_by_role_field taxonomy.
 *
 * @group entity_access_by_role_field
 */
class TaxonomyEntityAccessTests extends EntityAccessTestBase {
  use TaxonomyTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'entity_access_by_role_field',
    'taxonomy',
    'field',
  ];

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();

    // Create the adminUser:
    $this->createAdminRole('administrator', 'administrator');
    $this->adminUser = $this->createUser([
      'administer taxonomy',
      'access taxonomy overview',
    ]);
    $this->adminUser->addRole('administrator');

    // Create the global permissionUser:
    $this->globalPermissionUser = $this->createUser([
      'administer taxonomy',
      'access taxonomy overview',
      'entity_by_access_role_field.ignore_permission',
    ]);

    // Create the authenticatedUser:
    $this->authenticatedUser = $this->createUser([
      'administer taxonomy',
      'access taxonomy overview',
    ]);
    $this->authenticatedUser->addRole('authenticated_user');

    // Create the testRoleUser:
    $this->testRoleUser = $this->createUser([
      'administer taxonomy',
    ]);
    $this->createRole(['access taxonomy overview'], 'test_role', 'test_role');
    $this->testRoleUser->addRole('test_role');
  }

  /**
   * Check access on a taxonomy_term which allows authenticated users.
   */
  public function testAccessOnAllowedTaxonomyTerm() {
    $this->drupalLogin($this->authenticatedUser);
    $vocabularyBundle = $this->createVocabulary();
    $this->createAccessFieldTypeOnEntityBundle($vocabularyBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $term = $this->createTerm($vocabularyBundle);
    // Check expected access BEFORE we add our field:
    $this->drupalGet('/node/' . $term->id());
    $this->assertSession()->statusCodeEquals(200);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($term, $this->fieldName, ['authenticated_user'], Constants::ACCESS_ALLOWED);
    // Check expected access with our field:
    $this->drupalGet('taxonomy/term/' . $term->id());
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Check access on a taxonomy_term which forbids authenticated users.
   */
  public function testAccessOnForbiddenTaxonomyTerm() {
    $this->drupalLogin($this->authenticatedUser);
    $vocabularyBundle = $this->createVocabulary();
    $this->createAccessFieldTypeOnEntityBundle($vocabularyBundle, $this->fieldName, [Constants::OPERATION_VIEW], Constants::FALLBACK_NEUTRAL, FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
    $term = $this->createTerm($vocabularyBundle);
    // Check expected access BEFORE we add our field:
    $this->drupalGet('/node/' . $term->id());
    $this->assertSession()->statusCodeEquals(200);
    // Add our field:
    $this->setEntityAccessByRoleFieldOnEntity($term, $this->fieldName, ['authenticated_user'], Constants::ACCESS_FORBIDDEN);
    // Check expected access with our field:
    $this->drupalGet('taxonomy/term/' . $term->id());
    $this->assertSession()->statusCodeEquals(403);
  }

}
